//
//  SimpleXMLRPC.h
//  SimpleXMLRPC
//
//  Created by Markus Hutzler on 10/8/11.
//  Copyright 2011 spiderware gmbh. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface SimpleXMLRPC : NSObject
{
    NSURL * _url;
    NSString * _function;
    NSTimeInterval timeout;
    
    @private
    NSString * _content;
    id _delegate;
    id _data;
    
    // Request
    NSMutableData * receivedData;
    NSScanner * scanner;
}

// content holds the response string
@property (nonatomic, retain) NSString * content;
@property (nonatomic, retain) NSURL * url;
@property (nonatomic, retain) NSString * function;
@property NSTimeInterval timeout;

// This will init a SimpleXMLRPC object without a remote call 
- (id)initWithString:(NSString *)content delegate:(id)delegate;

// This will init a SimpleXMLRPC object with a remote call
- (id)initWithURL:(NSURL *)url function:(NSString *)function data:(id)data delegate:(id)delegate;

// makes the call
- (void) call;

// initialises the parse process
-(void) parseContent;

/* Scans the content-string for an object 
 
 boolean            <=> NSNumber
 int                <=> NSNumber
 i4                 <=> NSNumber
 double             <=> NSNumber
 base64             <=> NSData
 dateTime.iso8601   <=> NSDate
 string             <=> NSString
 array              <=> NSArray
 struct             <=> NSDictionary
 name               <=> NSString (used as key for NSDictionary)
 
 */
-(id) scanSuccess;                  // RPC Success
-(id) scanFault;                    // RPC Fault
-(id) scanValue;                    // Any Value see upper table
-(NSArray *) scanArray;
-(NSDictionary *) scanDictionary;
-(NSString *) scanName;

// raises an exception. reason: expected XML-Tag missing 
- (void) scanOrThrowException:(NSString *)tag;

// raises an exception. reason: not well formated XML
-(void) throwXMLRPCFormatException:(NSString *)reason;

@end



@protocol SimpleXMLRPCDelegate

/* SimpleXMLRPC will call a delegate to deliver the result to the referenced instance */

// Will be called if the server responds with a success.
-(void)simpleXMLRPC:(SimpleXMLRPC*)d didFinishWithData:(id)data;        

// Will be called if the server responds with an error.
-(void)simpleXMLRPC:(SimpleXMLRPC *)d didFailWithData:(id)data;

// Will be called if the connection to the server fails.
-(void)simpleXMLRPC:(SimpleXMLRPC *)d didFailWithError:(NSError*)error; 

@end
